from typing import Dict, List, Type, Union

from django.db.models import QuerySet
from rest_framework.response import Response
from rest_framework.serializers import Serializer


class ActionSerializersMixin:
    """
    Mixin for ViewSets that allows to have a dict with a serializer for
    each action.
    """

    action_serializers: Dict[str, Type[Serializer]] = {}

    def get_serializer_class(self):
        if hasattr(self, "action_serializers"):
            if self.action in self.action_serializers:
                return self.action_serializers[self.action]
        return super().get_serializer_class()


class SimpleSerializationMixin:
    """
    Mixin to simplify serialization of object instances.
    """

    def serialize_instance(self, data) -> Union[Dict, List]:
        serializer_class = self.get_serializer_class()  # type: ignore
        serializer = serializer_class(instance=data)

        return serializer.data


class SimplePaginationMixin:
    """
    Mixin to simplify pagination.
    """

    def paginate(self, queryset: QuerySet) -> Response:
        page = self.paginate_queryset(queryset)  # type: ignore
        if page is not None:
            serializer = self.get_serializer(page, many=True)  # type: ignore
            return self.get_paginated_response(serializer.data)  # type: ignore

        serializer = self.get_serializer(queryset, many=True)  # type: ignore
        return Response(serializer.data)


class SimpleValidationMixin:
    """
    Mixin to simplify validation.
    """

    def validate(self, data: dict, raise_exception=True) -> Serializer:
        serializer = self.get_serializer(data=data)  # type: ignore
        serializer.is_valid(raise_exception=raise_exception)
        return serializer



from functools import reduce
from typing import TYPE_CHECKING, Dict, List, Sequence, Tuple, Union

from django.db.models import CharField, Q, Value
from django.db.models.functions import Concat

if TYPE_CHECKING:
    from django.db.models import QuerySet


class SearchQuerySetMixin:
    """Mixin to add a searches method to a custom QuerySet."""

    def search(self, query: str) -> "QuerySet":
        """Search over the fields defined in the model."""
        if not query:
            return self  # Ignore the search if it's an empty sting
        try:
            fields: List[
                Union[Tuple[str, str], str]
            ] = self.model.SEARCH_FIELDS  # type: ignore
        except AttributeError:
            fields = []
        try:
            combined_fields: Dict[str, Sequence] = self.model.SEARCH_COMBINED_FIELDS  # type: ignore
        except AttributeError:
            combined_fields = {}
        conditions: List = []
        queryset: "QuerySet" = self
        if combined_fields:
            annotations = {}
            for name, combined_field in combined_fields.items():
                concat = []
                for item in combined_field:
                    concat += [item, Value(" ")]
                print(concat)
                annotations[name] = Concat(*concat, output_field=CharField())
            queryset = self.annotate(**annotations)  # type: ignore
        conditions += [
            Q(**{f"{field}__icontains": query})
            for field in fields + list(combined_fields.keys())
        ]
        if conditions:
            return queryset.filter(reduce(lambda x, y: x | y, conditions)).distinct()
        return self.none()  # type: ignore



from rest_framework.fields import Field, SerializerMethodField


class AnnotatedField(Field):
    """
    A read-only field that automatically returns an annotated field, taking
    into account a default value if the property is not found and if the user
    is present in the request
    """

    def __init__(self, default_value=None, requires_user=False, **kwargs):
        self.default_value = default_value
        self.requires_user = requires_user
        kwargs["source"] = "*"
        kwargs["read_only"] = True
        super().__init__(**kwargs)

    def get_value(self, instance):
        if self.requires_user and (
            "request" not in self.context
            or not self.context["request"].user.is_authenticated
        ):
            return self.default_value

        try:
            value = getattr(instance, self.field_name)
        except AttributeError:
            value = self.default_value

        return value

    def to_representation(self, value):
        return self.get_value(value)



from django.db.models.constants import LOOKUP_SEP
from django_filters import CharFilter
from rest_framework import filters


class SearchFilter(CharFilter):
    """Filter to use searches QuerySet method."""

    def filter(self, qs, value):
        if hasattr(qs, "search") and value:
            return qs.search(query=value)
        return qs


class UnaccentSearchFilter(filters.SearchFilter):
    """
    https://docs.djangoproject.com/en/3.1/ref/contrib/postgres/lookups/#unaccent
    You need migrate the postgres database and add 'django.contrib.postgres' in your INSTALLED_APPS
    ./manage.py makemigrations myapp --empty

    from django.contrib.postgres.operations import UnaccentExtension
    class Migration(migrations.Migration):
        operations = [
            UnaccentExtension()
        ]
    """

    def construct_search(self, field_name):
        lookup = "unaccent__icontains"
        return LOOKUP_SEP.join([field_name, lookup])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['belt', 'belt.django_filters', 'belt.rest_framework']

package_data = \
{'': ['*']}

install_requires = \
['django-filter>=21.1,<22.0',
 'django-model-utils>=4.1.0,<5.0.0',
 'django>=3.0.7,<4.0.0',
 'djangorestframework>=3.12.2,<4.0.0']

setup_kwargs = {
    'name': 'django-belt',
    'version': '1.8.0',
    'description': 'Simple package with some utilities for Django.',
    'long_description': '===========\nDjango Belt\n===========\n\nSimple package with some utilities for Django.\n\n.. image:: https://travis-ci.org/marcosgabarda/django-belt.svg?branch=master\n    :target: https://travis-ci.org/marcosgabarda/django-belt\n\n.. image:: https://coveralls.io/repos/github/marcosgabarda/django-belt/badge.svg?branch=master\n    :target: https://coveralls.io/github/marcosgabarda/django-belt?branch=master\n\n\nQuick start\n-----------\n\n**1** Install using pip::\n\n    pip install django-belt\n\n**2** Add "belt" to your INSTALLED_APPS settings like this::\n\n    INSTALLED_APPS += (\'belt\',)\n\n',
    'author': 'Marcos Gabarda',
    'author_email': 'hey@marcosgabarda.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/marcosgabarda/django-belt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
